@@ -65,7 +65,7 @@ class _ObjectList(vmmGObject):
             for obj in self._objects:
                 try:
                     obj.cleanup()
-                except:
+                except Exception:
                     logging.debug("Failed to cleanup %s", exc_info=True)
             self._objects = []
         finally:
@@ -264,6 +264,29 @@ class vmmConnection(vmmGObject):
     # Init routines #
     #################
 
+    def _wait_for_condition(self, compare_cb, timeout=3):
+        """
+        Wait for this object to emit the specified signal. Will not
+        block the mainloop.
+        """
+        from gi.repository import Gtk
+        is_main_thread = (threading.current_thread().name == "MainThread")
+        start_time = time.time()
+
+        while True:
+            cur_time = time.time()
+            if compare_cb():
+                return
+            if (cur_time - start_time) >= timeout:
+                return
+
+            if is_main_thread:
+                if Gtk.events_pending():
+                    Gtk.main_iteration_do(False)
+                    continue
+
+            time.sleep(.1)
+
     def _init_virtconn(self):
         self._backend.cb_fetch_all_guests = (
             lambda: [obj.get_xmlobj(refresh_if_nec=False)
@@ -281,20 +304,20 @@ class vmmConnection(vmmGObject):
                 for vol in pool.get_volumes():
                     try:
                         ret.append(vol.get_xmlobj(refresh_if_nec=False))
-                    except Exception, e:
+                    except Exception as e:
                         logging.debug("Fetching volume XML failed: %s", e)
             return ret
         self._backend.cb_fetch_all_vols = fetch_all_vols
 
-        def clear_cache(pools=False):
-            if not pools:
+        def cache_new_pool(obj):
+            if not self.is_active():
                 return
-
-            # This isn't synchronous, so any virtinst callers need to
-            # take that into account.
+            name = obj.name()
             self.schedule_priority_tick(pollpool=True)
-
-        self._backend.cb_clear_cache = clear_cache
+            def compare_cb():
+                return bool(self.get_pool(name))
+            self._wait_for_condition(compare_cb)
+        self._backend.cb_cache_new_pool = cache_new_pool
 
 
     ########################
@@ -348,13 +371,14 @@ class vmmConnection(vmmGObject):
     is_container = property(lambda s: getattr(s, "_backend").is_container)
     is_lxc = property(lambda s: getattr(s, "_backend").is_lxc)
     is_openvz = property(lambda s: getattr(s, "_backend").is_openvz)
+    is_vz = property(lambda s: getattr(s, "_backend").is_vz)
     is_xen = property(lambda s: getattr(s, "_backend").is_xen)
     is_remote = property(lambda s: getattr(s, "_backend").is_remote)
     is_qemu = property(lambda s: getattr(s, "_backend").is_qemu)
     is_qemu_system = property(lambda s: getattr(s, "_backend").is_qemu_system)
     is_qemu_session = property(lambda s:
                                getattr(s, "_backend").is_qemu_session)
-    is_test_conn = property(lambda s: getattr(s, "_backend").is_test)
+    is_test = property(lambda s: getattr(s, "_backend").is_test)
     is_session_uri = property(lambda s: getattr(s, "_backend").is_session_uri)
 
 
@@ -366,7 +390,7 @@ class vmmConnection(vmmGObject):
         uri = self.get_uri().replace("/", "_")
         ret = os.path.join(util.get_cache_dir(), uri)
         if not os.path.exists(ret):
-            os.makedirs(ret, 0755)
+            os.makedirs(ret, 0o755)
         return ret
 
     def get_default_storage_format(self):
@@ -440,6 +464,7 @@ class vmmConnection(vmmGObject):
         locals()[_supportname] = getattr(virtinst.VirtualConnection,
                                          _supportname)
     def check_support(self, *args):
+        # pylint: disable=no-value-for-parameter
         return self._backend.check_support(*args)
 
     def is_storage_capable(self):
@@ -549,7 +574,7 @@ class vmmConnection(vmmGObject):
                 try:
                     if vol.get_target_path() == path:
                         return vol
-                except Exception, e:
+                except Exception as e:
                     # Errors can happen if the volume disappeared, bug 1092739
                     logging.debug("Error looking up volume from path=%s: %s",
                         path, e)
@@ -624,7 +649,7 @@ class vmmConnection(vmmGObject):
         for dev in self.list_nodedevs():
             try:
                 xmlobj = dev.get_xmlobj()
-            except libvirt.libvirtError, e:
+            except libvirt.libvirtError as e:
                 # Libvirt nodedev XML fetching can be busted
                 # https://bugzilla.redhat.com/show_bug.cgi?id=1225771
                 if e.get_error_code() != libvirt.VIR_ERR_NO_NODE_DEVICE:
@@ -699,12 +724,12 @@ class vmmConnection(vmmGObject):
         try:
             # Redefine new domain
             newobj = define_cb(newxml)
-        except Exception, renameerr:
+        except Exception as renameerr:
             try:
                 logging.debug("Error defining new name %s XML",
                     obj.class_name(), exc_info=True)
                 newobj = define_cb(origxml)
-            except Exception, fixerr:
+            except Exception as fixerr:
                 logging.debug("Failed to redefine original %s!",
                     obj.class_name(), exc_info=True)
                 raise RuntimeError(
@@ -839,7 +864,7 @@ class vmmConnection(vmmGObject):
                 self._domain_lifecycle_event, None))
             self.using_domain_events = True
             logging.debug("Using domain events")
-        except Exception, e:
+        except Exception as e:
             self.using_domain_events = False
             logging.debug("Error registering domain events: %s", e)
 
@@ -850,7 +875,7 @@ class vmmConnection(vmmGObject):
                 self._domain_cb_ids.append(
                     self.get_backend().domainEventRegisterAny(
                     None, eventid, self._domain_xml_misc_event, None))
-            except Exception, e:
+            except Exception as e:
                 logging.debug("Error registering domain %s event: %s",
                     typestr, e)
 
@@ -876,7 +901,7 @@ class vmmConnection(vmmGObject):
                 None, eventid, self._network_lifecycle_event, None))
             self.using_network_events = True
             logging.debug("Using network events")
-        except Exception, e:
+        except Exception as e:
             self.using_network_events = False
             logging.debug("Error registering network events: %s", e)
 
@@ -896,7 +921,7 @@ class vmmConnection(vmmGObject):
                 None, refreshid, self._storage_pool_refresh_event, None))
             self.using_storage_pool_events = True
             logging.debug("Using storage pool events")
-        except Exception, e:
+        except Exception as e:
             self.using_storage_pool_events = False
             logging.debug("Error registering storage pool events: %s", e)
 
@@ -915,7 +940,7 @@ class vmmConnection(vmmGObject):
 
             self.using_node_device_events = True
             logging.debug("Using node device events")
-        except Exception, e:
+        except Exception as e:
             self.using_network_events = False
             logging.debug("Error registering node device events: %s", e)
 
@@ -943,7 +968,7 @@ class vmmConnection(vmmGObject):
                     self._backend.storagePoolEventDeregisterAny(eid)
                 for eid in self._node_device_cb_ids:
                     self._backend.nodeDeviceEventDeregisterAny(eid)
-        except:
+        except Exception:
             logging.debug("Failed to deregister events in conn cleanup",
                 exc_info=True)
         finally:
@@ -972,7 +997,7 @@ class vmmConnection(vmmGObject):
         self._backend.cb_fetch_all_pools = None
         self._backend.cb_fetch_all_nodedevs = None
         self._backend.cb_fetch_all_vols = None
-        self._backend.cb_clear_cache = None
+        self._backend.cb_cache_new_pool = None
 
     def open(self):
         if not self.is_disconnected():
@@ -987,7 +1012,7 @@ class vmmConnection(vmmGObject):
     def _do_creds_password(self, creds):
         try:
             return connectauth.creds_dialog(self, creds)
-        except:
+        except Exception:
             logging.debug("Launching creds dialog failed", exc_info=True)
             return -1
 
@@ -999,7 +1024,7 @@ class vmmConnection(vmmGObject):
         try:
             self._backend.open(self._do_creds_password)
             return True, None
-        except Exception, exc:
+        except Exception as exc:
             tb = "".join(traceback.format_exc())
             if isinstance(exc, libvirt.libvirtError):
                 # pylint: disable=no-member
@@ -1042,17 +1067,17 @@ class vmmConnection(vmmGObject):
         # We want this before events setup to save some needless polling
         try:
             virtinst.StoragePool.build_default_pool(self.get_backend())
-        except Exception, e:
+        except Exception as e:
             logging.debug("Building default pool failed: %s", str(e))
 
         self._add_conn_events()
 
         # Prime CPU cache
-        self.caps.get_cpu_values("x86_64")
+        self.caps.get_cpu_values(self.caps.host.cpu.arch)
 
         try:
             self._backend.setKeepAlive(20, 1)
-        except Exception, e:
+        except Exception as e:
             if (type(e) is not AttributeError and
                 not util.is_error_nosupport(e)):
                 raise
@@ -1084,7 +1109,7 @@ class vmmConnection(vmmGObject):
 
             self.idle_add(self._change_state, is_active and
                 self._STATE_ACTIVE or self._STATE_DISCONNECTED)
-        except Exception, e:
+        except Exception as e:
             is_active = False
             self._schedule_close()
             connectError = (str(e), "".join(traceback.format_exc()), False)
@@ -1111,7 +1136,7 @@ class vmmConnection(vmmGObject):
             class_name = obj.class_name()
             try:
                 name = obj.get_name()
-            except:
+            except Exception:
                 name = str(obj)
 
             if not self._objects.remove(obj):
@@ -1132,7 +1157,7 @@ class vmmConnection(vmmGObject):
                 self.emit("nodedev-removed", obj.get_connkey())
             obj.cleanup()
 
-    def _new_object_cb(self, obj, initialize_failed):
+    def _new_object_cb(self, obj, initialize_failed, skip_init=False):
         if not self._backend.is_open():
             return
 
@@ -1165,7 +1190,7 @@ class vmmConnection(vmmGObject):
             elif class_name == "nodedev":
                 self.emit("nodedev-added", obj.get_connkey())
         finally:
-            if self._init_object_event:
+            if self._init_object_event and not skip_init:
                 self._init_object_count -= 1
                 if self._init_object_count <= 0:
                     self._init_object_event.set()
@@ -1310,7 +1335,7 @@ class vmmConnection(vmmGObject):
                     continue
 
                 obj.tick(stats_update=stats_update)
-            except Exception, e:
+            except Exception as e:
                 logging.exception("Tick for %s failed", obj)
                 if (isinstance(e, libvirt.libvirtError) and
                     (getattr(e, "get_error_code")() ==
@@ -1399,7 +1424,7 @@ class vmmConnection(vmmGObject):
             self._tick(*args, **kwargs)
         except KeyboardInterrupt:
             raise
-        except Exception, e:
+        except Exception as e:
             pass
 
         if e is None:
